@@ -2,9 +2,10 @@
 # this repository contains the full copyright notices and license terms.
 import datetime
 from collections import deque, defaultdict
+from functools import reduce
 from heapq import heappop, heappush
 
-from trytond.model import ModelSQL, fields
+from trytond.model import ModelSQL, fields, tree
 from trytond.wizard import Wizard, StateTransition
 from trytond.pyson import Eval
 from trytond.transaction import Transaction
@@ -17,8 +18,7 @@ def intfloor(x):
     return int(round(x, 4))
 
 
-class Work:
-    __metaclass__ = PoolMeta
+class Work(tree(parent='successors'), metaclass=PoolMeta):
     __name__ = 'project.work'
     predecessors = fields.Many2Many('project.predecessor_successor',
         'successor', 'predecessor', 'Predecessors',
@@ -74,11 +74,6 @@ class Work:
     def __setup__(cls):
         super(Work, cls).__setup__()
 
-    @classmethod
-    def validate(cls, works):
-        super(Work, cls).validate(works)
-        cls.check_recursion(works, parent='successors')
-
     @staticmethod
     def default_leveling_delay():
         return 0.0
@@ -435,7 +430,7 @@ class Work:
         write_fields = ('early_start_time', 'early_finish_time',
                         'late_start_time', 'late_finish_time')
         to_write = []
-        for work, val in values.iteritems():
+        for work, val in values.items():
             write_cond = False
             for field in write_fields:
                 if field in val and getattr(work, field) != val[field]:
@@ -451,9 +446,10 @@ class Work:
         get_key = lambda w: (set(p.id for p in w.predecessors),
                              set(s.id for s in w.successors))
 
-        parent_id = self.parent and self.parent.id or None
+        if not self.parent:
+            return
         siblings = self.search([
-                ('parent', '=', parent_id)
+                ('parent', '=', self.parent.id)
                 ])
         to_clean = []
 
@@ -596,7 +592,7 @@ class PredecessorSuccessor(ModelSQL):
         super(PredecessorSuccessor, cls).write(*args)
 
         work_ids = [v for values in args[1::2]
-            for k, v in values.itervalues()
+            for k, v in values.values()
             if k in ('predecessor', 'successor')]
         works = Work.browse(work_ids)
         for work in works:
